import React, { useState, useRef } from 'react'
import TrashIcon, { LinearIcon, RadialIcon, MenuIcon, DropperIcon } from './icon'
import { usePicker } from './context'
import html2canvas from 'html2canvas'
import Portal from 'robofox-react-portal';

var tinycolor = require("tinycolor2");

const Controls = () => {
  const { isGradient, onChange, gradientType, setSelectedColor, colors, currentColor } = usePicker();
  const [openAdvanced, setOpenAdvanced] = useState(false);
  const isLinear = gradientType === 'linear-gradient';
  const [inputType, setInputType] = useState('rgb');

  const setSolid = () => {
    setSelectedColor(0)
    onChange('rgba(175, 51, 242, 1)')
  }

  const setGradient = () => {
    onChange('linear-gradient(90deg, rgba(2,0,36,1) 0%, rgba(9,9,121,1) 35%, rgba(0,212,255,1) 100%)')
  }

  const lighten = (value) => {
    let newC = tinycolor(currentColor).lighten(value).toRgb()
    console.log(newC);
  }

  const brighten = (value) => {
    let newC = tinycolor(currentColor).brighten(value).toRgb()
    console.log(newC);
  }

  const darken = (value) => {
    let newC = tinycolor(currentColor).darken(value).toRgb()
    console.log(newC);
  }

  const desaturate = (value) => {
    let newC = tinycolor(currentColor).desaturate(value).toRgb()
    console.log(newC);
  }

  const saturate = (value) => {
    let newC = tinycolor(currentColor).saturate(value).toRgb()
    console.log(newC);
  }

  const analogous = () => {
    let colors = tinycolor(currentColor).analogous()
    console.log(colors);
  }

  const monochromatic = () => {
    let colors = tinycolor(currentColor).monochromatic()
    console.log(colors);
  }

  const triad = () => {
    let colors = tinycolor(currentColor).triad()
    console.log(colors);
  }

  const tetrad = () => {
    let colors = tinycolor(currentColor).tetrad()
    console.log(colors);
  }
  // console.log(colors);
  // {(isLinear && isGradient) && <DegreePicker />}
  // {isGradient && <GradientType />}
  // {(isGradient && colorz.length > 2) && <DeleteBtn />}



  return(
    <div style={{paddingTop: 12, paddingBottom: 9, overflow: 'hidden', }}>
      <div  className='df jsb ac' style={{width: '100%'}}>
        <EyeDropper />
        <div className='df jc ac' style={{height: 28, background: '#e9e9f5', borderRadius: 6, padding: 2}}>
          <div style={controlBtnStyles(!isGradient)} className='control-btn df ac' onClick={setSolid}>Solid</div>
          <div style={controlBtnStyles(isGradient)} className='control-btn df ac' onClick={setGradient}>Gradient</div>
        </div>
        <div className='df ac jfe' style={{height: 28, background: '#e9e9f5', borderRadius: 6, padding: 2}}>
          <div style={{width: 24}} className='df jc' onClick={() => setOpenAdvanced(!openAdvanced)}>
            <MenuIcon />
          </div>
        </div>
      </div>
      <div style={{height: openAdvanced ? 200 : 0, width: '100%', transition: 'all 120ms linear'}}>
        <div style={{paddingTop: 11, opacity: openAdvanced ? 1 : 0}}>
          <div className='df jc ac'>
            <div className='df ac' style={{height: 28, background: '#e9e9f5', borderRadius: 6, padding: 2}}>
              <div style={controlBtnStyles(inputType === 'hsl')} className='control-btn df ac' onClick={() => setInputType('hsl')}>HSL</div>
              <div style={controlBtnStyles(inputType === 'rgb')} className='control-btn df ac' onClick={() => setInputType('rgb')}>RGB</div>
              <div style={controlBtnStyles(inputType === 'hsv')} className='control-btn df ac' onClick={() => setInputType('hsv')}>HSV</div>
            </div>
          </div>
        </div>
      </div>
    </div>
  )
}

export default Controls

const GradientType = () => {
  const { gradientType, onChange, value } = usePicker()

  const handleLinear = () => {
    const remaining = value.split(/,(.+)/)[1]
    onChange(`linear-gradient(90deg, ${remaining}`)
  }

  const handleRadial = () => {
    const remaining = value.split(/,(.+)/)[1]
    onChange(`radial-gradient(circle, ${remaining}`)
  }

  return(
    <div className='df ac' style={{marginRight: 11}}>
      <div className='control-grad-wrap' onClick={handleLinear} style={{marginRight: 1, ...gradBtnStyles(gradientType==='linear-gradient')}}>
        <LinearIcon />
      </div>
      <div className='control-grad-wrap' onClick={handleRadial} style={{marginLeft: 1, ...gradBtnStyles(gradientType==='radial-gradient')}}>
        <RadialIcon />
      </div>
    </div>
  )
}

const DegreePicker = () => {
  const { degrees, value, onChange } = usePicker()

  const handleDegrees = (e) => {
    let num = parseInt(e.target.value)
    let nans = isNaN(num) ? 0 : num
    let min = Math.max(nans, 0)
    let max = Math.min(min, 360)
    const remaining = value.split(/,(.+)/)[1]
    onChange(`linear-gradient(${max}deg, ${remaining}`)
  }

  return(
    <div className='ps-rl' style={{marginRight: 14}}>
      <input className='degree-input' value={degrees} onChange={(e) => handleDegrees(e)}/>
      <div style={{position: 'absolute', right: degrees > 99 ? 0 : 3, top: -1, fontWeight: 400, fontSize: 13}}>°</div>
    </div>
  )
}

const DeleteBtn = () => {
  const { deletePoint } = usePicker();

  return(
    <div onClick={deletePoint} className='delete-point-btn df jc ac'>
      <TrashIcon />
    </div>
  )
}

const gradBtnStyles = (selected) => {
  return{
    background: selected ? 'white' : '',
    boxShadow: selected ? '1px 1px 6px rgba(0,0,0,.15)' : ''
  }
}

const controlBtnStyles = (selected) => {
  return{
    background: selected ? 'white' : '',
    color: selected ? '#568CF5' : '',
    boxShadow: selected ? '0px 0px 8px rgba(0,0,0,.125)' : ''
  }
}

const EyeDropper = () => {
  const { handleChange } = usePicker();
  const [pickerCanvas, setPickerCanvas] = useState('')
  const [coverUp, setCoverUp] = useState(false)
  const [eyeDrop, setEyeDrop] = useState('')

  const takePick = () => html2canvas(document.body)
  .then((canvas) => {
    const croppedCanvas = document.createElement('canvas')
    const croppedCanvasContext = croppedCanvas.getContext('2d')

    const cropPositionTop = 0
    const cropPositionLeft = 0
    const cropWidth = window.innerWidth * 2
    const cropHeight = window.innerHeight * 2
    croppedCanvas.width = cropWidth
    croppedCanvas.height = cropHeight

    croppedCanvasContext.drawImage(
     canvas,
     cropPositionLeft,
     cropPositionTop,
   )

    setPickerCanvas(croppedCanvasContext)
    setCoverUp(true)
  })

  const getEyeDrop = (e) => {
    e.stopPropagation()
    const x1 = e.clientX * 2;
    const y1 = e.clientY * 2;
    const [r, g, b] = pickerCanvas.getImageData(x1, y1, 1, 1).data;
    handleChange(`rgba(${r}, ${g}, ${b}, 1)`)
    setCoverUp(false)
  }

  return(
    <div className='df ac jfe' style={{height: 28, background: '#e9e9f5', borderRadius: 6, padding: 2, cursor: 'copy'}}>
      <div style={{width: 24}} className='df jc' onClick={takePick}>
        <DropperIcon />
      </div>
      {coverUp &&
        <Portal id="eyeDropCover">
          <div style={{ position: 'absolute', left: 0, top: 0, zIndex: 100000000, width: window.innerWidth, height: window.innerHeight }} onClick={(e) => getEyeDrop(e)} />
        </Portal>
      }
    </div>
  )
}
